package org.vraptor.atx;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.objectweb.asm.ClassReader;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Ant task which translates annotations into a new vraptor xml file.
 * @since 1.0
 */
public class AnnotationsToXmlTask extends Task {

    /**
     * Location of the file.
     * @required
     */
    private File outputDirectory;

    public void execute() throws BuildException {
        try {
            File f = outputDirectory;
            if (!f.exists()) {
                f.mkdirs();
            }
            log("Base directory is " + f.getAbsolutePath());
            List<String> components = new ArrayList<String>();
            searchInside(f, "", components);
            FileWriter fw = new FileWriter(new File(f, "vraptor-vatx.xml"));
            PrintWriter writer = new PrintWriter(fw);
            writer.println("<vraptor>");
            for (String component : components) {
                writer.printf("\t<component>%s</component>\n", component);
            }
            writer.println("\t<autoDiscover>false</autoDiscover>");
            writer.println("</vraptor>");
            writer.close();
            fw.close();
        } catch (IOException ex) {
            throw new BuildException("Unable to write vraptor xml file.", ex);
        }
    }

    private void searchInside(File f, String current, List<String> list) throws IOException {
        log("Searching " + f.getAbsolutePath() + " : " + current);
        for (File child : f.listFiles()) {
            if (child.isDirectory()) {
                searchInside(child, current + child.getName() + ".", list);
            } else {
                if (child.getName().endsWith(".class")) {
                    FileInputStream is = new FileInputStream(child);
                    log("Processing " + child.getAbsolutePath());
                    ClassReader reader = new ClassReader(is);
                    TypeCollector visitor = new TypeCollector();
                    reader.accept(visitor, 0);
                    if (visitor.isComponentAnnotationFound()) {
                        String name = current + child.getName();
                        name = name.substring(0, name.lastIndexOf("."));
                        log("Found type " + name);
                        list.add(name);
                    }
                }
            }
        }
    }

    public void setOutputDirectory(File outputDirectory) {
        this.outputDirectory = outputDirectory;
    }
}
